package lc.esp.cli.sdk;

import lc.esp.sdk.*;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;

import java.util.List;
import java.util.UUID;

/**
 * This class implements a simple client for sending requests to a {@link CLIService}.
 *
 * @author Alex Leigh
 * @since 18.0
 */
public class CLIClient extends BasicallyDangerous {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(CLIClient.class);
    private final ESPClient client;
    private final ESPAddress address;

    public CLIClient(ESPClient client, ESPAddress address) {
        this.client = client;
        this.address = address;
    }

    public CLIResponse exec(List<String> cmd) throws Exception {
        ESPAddress src = new ESPAddress("lc", "client", "cli-client",
                UUID.randomUUID().toString(), ESPAddressClass.QUEUE, ESPMessageClass.COMMAND);

        logger.info("Received exec(). [cmd: {}]", cmd);

        try (ESPSession session = client.createSession()) {
            ESPConsumer consumer = session.createConsumer(src);
            ESPProducer producer = session.createProducer(address);
            CLIRequest req = new CLIRequest(cmd);
            ESPMessage reqMsg = req.toMessage();
            reqMsg.setReplyTo(src);
            producer.send(reqMsg);
            logger.info("Waiting for reply...");
            return new CLIResponse(consumer.receive());
            // We don't close the producer/consumer on the premise that the session did that
        }
    }

    @Override
    public void runDangerously() throws Exception {
        client.start();
    }
}
