package lc.esp.cli.sdk;

import lc.esp.sdk.ESPAddress;
import lc.esp.sdk.ESPClient;
import lc.esp.sdk.ESPMessage;
import lc.esp.sdk.service.ESPService;
import lc.mecha.cli.cmd.BasicCommandProcessor;
import lc.mecha.cli.line.BasicLineHandler;
import lc.mecha.util.StringAccumulatorV2;

import java.util.List;

/**
 * This class implements the framework for a CLI-based service. It formats the messages in accordance with
 * {@link CLIRequest} and {@link CLIResponse}. This CLI implementation is based on lc-mecha-cli, but the
 * underlying transport could be used for arbitrary command execution frameworks, including remote shell
 * command execution.
 *
 * @author Alex Leigh
 * @since 18.1
 */
public abstract class CLIService extends ESPService {
    private final BasicCommandProcessor bcp = new BasicCommandProcessor();

    public CLIService(ESPClient client, ESPAddress address) {
        super(client, address);
    }

    public BasicCommandProcessor getBcp() {
        return bcp;
    }

    public CLIService(ESPAddress address) throws Exception {
        super(address);
    }

    public CLIService(String address) throws Exception {
        super(address);
    }

    @Override
    public ESPMessage handle(ESPMessage msg) throws Exception {
        CLIRequest req = new CLIRequest(msg);
        List<String> cmd = req.getCmd();
        StringAccumulatorV2 outBuf = new StringAccumulatorV2(" ");

        bcp.handleCommand(new BasicLineHandler() {
            @Override
            public void write(String s, boolean doPrompt) {
                outBuf.add(s);
            }

            @Override
            public void close() {

            }
        }, cmd.toArray(new String[0]));

        CLIResponse res = new CLIResponse(outBuf.asString(), null);
        return res.toMessage();
    }
}