package lc.esp.sdk.service;

import lc.esp.sdk.*;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;

/**
 * This class implements a basic ESP request/response server.
 *
 * @author Alex Leigh
 * @since 18.1
 */
public abstract class ESPService implements Runnable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(ESPService.class);
    private final ESPClient client;
    private final ESPAddress address;

    public ESPService(ESPClient client, ESPAddress address) {
        this.client = client;
        this.address = address;
    }

    public ESPService(ESPAddress address) throws Exception {
        client = new ESPClient();
        client.start();
        this.address = address;
    }

    public ESPService(String address) throws Exception {
        this(new ESPAddress(address));
    }

    @Override
    public void run() {
        try {
            try (ESPSession session = client.createSession()) {
                try (ESPConsumer consumer = session.createConsumer(address)) {
                    //noinspection InfiniteLoopStatement
                    while (true) {
                        ESPMessage request = consumer.receive();
                        ESPAddress replyAddress = request.getReplyTo();
                        if (replyAddress != null) {
                            try (ESPProducer producer = session.createProducer(replyAddress)) {
                                ESPMessage reply = handle(request);
                                if (reply != null) {
                                    // We force the correlation ID to match the request.
                                    String correlationId = request.getCorrelationId();
                                    if (correlationId != null) {
                                        reply.setCorrelationId(correlationId);
                                    }
                                    producer.send(reply);
                                }
                            }
                        } else {
                            logger.warn("Caller did not supply a return address: {}", request);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public abstract ESPMessage handle(ESPMessage msg) throws Exception;
}
