package lc.hello.app;

import lc.esp.sdk.*;
import lc.mecha.json.JSONObject;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;

import java.util.UUID;

public class HelloApp {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(HelloApp.class);

    public static void main(String[] args) throws Exception {
        try (ESPClient esp = new ESPClient()) {
            esp.start();

            ESPAddress src = new ESPAddress("lc", "dev", "hello-client",
                    UUID.randomUUID().toString(), ESPAddressClass.QUEUE, ESPMessageClass.COMMAND);
            ESPAddress dst = new ESPAddress("queue://lc.dev.example.hello.cmd");

            try (ESPSession session = esp.createSession()) {
                ESPConsumer consumer = session.createConsumer(src);
                ESPProducer producer = session.createProducer(dst);
                ESPMessage request = new ESPMessage(new JSONObject().put("name", "Alex"));
                request.setCorrelationId(UUID.randomUUID().toString());
                request.setReplyTo(src);
                producer.send(request);
                logger.info("Waiting for reply...");
                ESPMessage msg = consumer.receive();
                logger.info("Received reply: {}", msg);
            }
        }
    }
}
